import network
import socket
import time
import json
from machine import I2C, Pin
from machine_i2c_lcd import I2cLcd

#Zugangsdaten zum WLAN Netzwerk
ssid = '******'
password = '*****'

#LCD-Display
lines = 2
cols = 16

i2c = I2C(0, sda=Pin(0), scl=Pin(1), freq=100000)
lcd = I2cLcd(i2c, 0x27, lines, cols)

#Adresse welche uns das JSON mit den Zeitdaten liefert
service = 'http://worldtimeapi.org/api/timezone/'
url = service + 'Europe/Berlin'

# Aufbau einer WiFi Verbindung
def wifiConnect():
    wlan = network.WLAN(network.STA_IF)
    wlan.active(True)
    wlan.connect(ssid, password)

    print("Waiting to connect:")
    while not wlan.isconnected() and wlan.status() >= 0:
        print(".", end="")   
        time.sleep(1)
    print("")
    print(wlan.ifconfig())

# Funktion zum absenden eines HTTP Request und
# Rückgabe des HTTP Response
# Quelle: https://docs.micropython.org/en/latest/esp8266/tutorial/network_tcp.html
def http_get(url):
    result = ''
    _, _, host, path = url.split('/', 3)
    addr = socket.getaddrinfo(host, 80)[0][-1]
    s = socket.socket()
    s.connect(addr)
    s.send(bytes('GET /%s HTTP/1.0\r\nHost: %s\r\n\r\n' % (path, host), 'utf8'))
    while True:
        data = s.recv(100)
        if data:
            result = result + str(data, 'utf8')
        else:
            break
    s.close()
    return result

# Ermitteln des JSONs aus dem HTTP Response
def findJson(response):
    txt = 'abbreviation'
    return response[response.find(txt)-2:]

# parsen des Zeitstempels
# Parameter ist das JSON als Dictionary
def parseDateTimeStr(responeDict):
    # umwandeln des UNIX Timestamp in eine Liste aus Datum & Zeit Werten
    dateTime = time.localtime(int(responeDict['unixtime']))
    year = dateTime[0]
    month = dateTime[1]
    dayOfMonth = dateTime[2]
    
    # Wenn der Monat kleiner als 10 ist,
    # dann eine führende Null anhängen
    if month < 10:
        month = str('0' + str(month))
        
    if dayOfMonth < 10:
        dayOfMonth = str('0' + str(dayOfMonth))

    dateStr = str(dayOfMonth)+'.'+str(month)+'.'+str(year)

    hour = dateTime[3]
    minutes = dateTime[4]

    # offset für die Uhrzeit auslesen
    timeOffset = responeDict['utc_offset']
    # Wenn der Offset mit einem Minus beginnt
    # dann soll der Wert abgezogen werden
    if timeOffset[0:1] == '-':
        hour = hour - int(timeOffset[1:3])
    elif timeOffset[0:1] == '+':
        # beginnt der Offset mit einem "+"
        # dann soll die Zeit addiert werden
        hour = hour + int(timeOffset[1:3])

    if hour < 10:
        hour = str('0' + str(hour))

    if minutes < 10:
        minutes = str('0' + str(minutes))
    
    timeStr = str(hour)+':'+str(minutes)
    # auslesen der Zeitzone
    timezone = responeDict['timezone']
# zurückgeben der Zeitzone, des Datums sowie der Uhrzeit
    return timezone, dateStr, timeStr

# Anzeigen der Daten auf dem Display
# Hinweis: Wenn die Zeichen der Zeitzone
# länger als 16 Zeichen ist, dann wird
# diese automatisch umgebrochen!
def displayDateTime(timezone, dateStr, timeStr):
    timezone = "{:<16}".format(timezone)
    lcd.putstr(timezone +"\n" +dateStr + " " + timeStr)
    time.sleep(2)

# Main Funktion
def main():
    # Aufbau der Wifi-Verbindung
    wifiConnect()
    # Auslesen des HTTP Response
    response = http_get(url)
    # ermitteln des JSONs
    jsonData = findJson(response)
    # umwandeln des JSONs in ein Dictionary
    aDict = json.loads(jsonData)
    # parsen des Zeitstempels aus dem Dictionary
    timezone, dateStr, timeStr = parseDateTimeStr(aDict)
    # ausgeben der Werte auf dem LCD-Display
    displayDateTime(timezone, dateStr, timeStr)

# Aufrufen der Funktion main()
while True:
    main()
    # eine Pause von 60 Sekunden einlegen
    # Der Server leht die Verbindung ab wenn der Intervall
    # des Zugriffs zu klein ist.
    time.sleep(60)
